#ifndef __EKAUQM_MAP__
#define __EKAUQM_MAP__

#include "includes.h"
#include "constantes.h"
#include "dessin.h"
#include "math.h"
#include "script.h"
#include "sdlglutils.h"
#include "perso.h"
#include "balle.h"

///////////////////////////////
// Definitions de variables: //
///////////////////////////////

// Plafon:
#define PLAFON_MAX					512
#define PLAFON_TEXTURE				0
#define PLAFON_TEXTURE_REPETITION	1
#define PLAFON_HAUTEUR				2
#define PLAFON_X1					3
#define PLAFON_Y1					4
#define PLAFON_X2					5
#define PLAFON_Y2					6
#define PLAFON_PARAMETRE_NB			7

// Mur
#define MUR_DISTANCE_MIN		0.35
#define MUR_MAX					512
#define MUR_TEXTURE				0
#define MUR_TEXTURE_REPETITION	1
#define MUR_TYPE				2
#define MUR_XY_POS				3
#define MUR_XY_MIN				4
#define MUR_XY_MAX				5
#define MUR_Z_MIN				6
#define MUR_Z_MAX				7
#define MUR_PARAMETRE_NB		8

// Porte
#define PORTE_OUVERTURE_MAX	0.2
#define PORTE_VITESSE		0.01
#define PORTE_MAX			128
#define PORTE_TEXTURE		0
#define PORTE_TYPE			1
#define PORTE_XY_POS		2
#define PORTE_XY_MIN		3
#define PORTE_XY_MAX		4
#define PORTE_Z_MIN			5
#define PORTE_Z_MAX			6
#define PORTE_HAUTEUR_VAR	7
#define PORTE_PRECEDANT		8
#define PORTE_PARAMETRE_NB	9

#define PORTE_OUVRE			0
#define PORTE_FERME			1
#define PORTE_RIEN			2
#define DISTANCE_ECOUTE_PORTE_MINIMUM 10*10*UNITE*UNITE

// Apparition
#define APPARITION_MAX			256
#define APPARITION_X			0
#define APPARITION_Y			1
#define APPARITION_PARAMETRE_NB	2
#define APPARITION_DEFAUT_X		1
#define APPARITION_DEFAUT_Y		1

// Teleporteur
#define TELEPORTEUR_MAX				256
#define TELEPORTEUR_TEXTURE			0
#define TELEPORTEUR_SENS			1
#define TELEPORTEUR_POS_X			2
#define TELEPORTEUR_POS_Y			3
#define TELEPORTEUR_D_X				4
#define TELEPORTEUR_D_Y				5
#define TELEPORTEUR_ANGLE			6
#define TELEPORTEUR_PARAMETRE_NB	7
#define TELEPORTEUR_DEFAUT_D_X		1
#define TELEPORTEUR_DEFAUT_D_Y		1
#define TELEPORTEUR_LARGEUR			0.5
#define TELEPORTEUR_HAUTEUR			2

// Caisse
#define CAISSE_MAX					128
#define CAISSE_VIE					15
#define CAISSE_TAILLE				0.2
#define CAISSE_DECALAGE				0.5 // ce qu'on rajoute au coordonnes pour que les caisse soient au milieu d'un carr
#define CAISSE_TEMPS_REAPARITION	30000 // en ms
#define CAISSE_EST_NON_RAMASSEE		0
#define CAISSE_EST_RAMASSEE			1
#define CAISSE_TEXTURE				0
#define CAISSE_POS_X				1
#define CAISSE_POS_Y				2
#define CAISSE_TEMPS_RESTANT		3
#define CAISSE_RAMASSEE				4
#define CAISSE_PARAMETRE_NB			5

// Textures:
#define TEXTURE_MUR_TAILLE_DEFAUT		1
#define TEXTURE_PLAFON_TAILLE_DEFAUT	1
#define TEXTURE_AUCUNE					"rien"


/////////////////////////////
// Definitions de classes: //
/////////////////////////////

class map
{
private:
	// Textures:
	GLuint ListeDesTextures[LISTE_NOMBRE_MAX];
	char ListeDesNomsDeTextures[LISTE_NOMBRE_MAX][LISTE_NOM_TAILLE_MAX];
	int NombreDeTextures;

	int MapChargee;

	// Decors:
	int NbPlafon;
	int Plafon[PLAFON_MAX][PLAFON_PARAMETRE_NB];
	int NbMur;
	int Mur[MUR_MAX][MUR_PARAMETRE_NB];
	int NbApparition;
	int Apparition[APPARITION_MAX][APPARITION_PARAMETRE_NB];
	int NbTeleporteur;
	int Teleporteur[TELEPORTEUR_MAX][TELEPORTEUR_PARAMETRE_NB];
	int NbPorte;
	int Porte[PORTE_MAX][PORTE_PARAMETRE_NB];
	int NbCaisse;
	int Caisse[CAISSE_MAX][CAISSE_PARAMETRE_NB];

public:
	void InitialisationMap();
	int InitialisationTextures(int NumMap);
	int ChargeMap(char* Nom, bool Charge, int NumMap);

	void CalculColision(double &X, double &Y, double PosX, double PosY);
	bool CalculTeleporteur(double &X, double &Y, double &PosX, double &PosY,double &Angle);
	int CalculOuverturePorte(int N,double *X,double *Y,int msPasse);
	int CalculRamassageCaisse(double X,double Y,double &Vie,int Duree);
	void RamasseCaisse(int N);
	void CalculColisionBalle(balle *Balle,double &Vx,double &Vy,double &Vz);

	void ObtenirApparition(POINT* apparition);

	void Dessin();
	void DessinerLesPlafons();
	void DessinerLesMurs();
	void DessinerLesCaisses();
	void DessinerLesTeleporteurs();
	void DessinerLesPortes();
};

#endif 